@@ -24,6 +24,7 @@
 
 #include <config.h>
 
+#include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -51,8 +52,9 @@ static int inp_alsa_cleanup (VisPluginData *plugin);
 static int inp_alsa_upload (VisPluginData *plugin, VisAudio *audio);
 
 static const int  inp_alsa_var_btmul      = sizeof (short);
-static const char *inp_alsa_var_cdevice   = "hw:0,0";
+static const char *inp_alsa_var_cdevice   = "default";
 static const int  inp_alsa_var_samplerate = 44100;
+static const int  inp_alsa_var_frames_target = 256;
 static const int  inp_alsa_var_channels   = 2;
 
 VISUAL_PLUGIN_API_VERSION_VALIDATOR
@@ -106,7 +108,7 @@ int inp_alsa_init (VisPluginData *plugin)
 
 	visual_object_set_private (VISUAL_OBJECT (plugin), priv);
 
-	if ((err = snd_pcm_open(&priv->chandle, strdup(inp_alsa_var_cdevice),
+	if ((err = snd_pcm_open(&priv->chandle, inp_alsa_var_cdevice,
 			SND_PCM_STREAM_CAPTURE, SND_PCM_NONBLOCK)) < 0) {
 		visual_log(VISUAL_LOG_CRITICAL, 
 			    _("Record open error: %s"), snd_strerror(err));
@@ -165,16 +167,14 @@ int inp_alsa_init (VisPluginData *plugin)
 		return(-1);
 	}
 
-       /* Setup a large buffer */
-
-	tmp = 1000000;
+	tmp = (int)ceil((1000.0 * 1000.0 * inp_alsa_var_frames_target) / inp_alsa_var_samplerate);  // in micro seconds
 	if (snd_pcm_hw_params_set_period_time_near(priv->chandle, hwparams, &tmp, &dir) < 0){
 		visual_log(VISUAL_LOG_CRITICAL, _("Error setting period time"));
 		snd_pcm_hw_params_free(hwparams);
 		return(-1);
 	}
 
-	tmp = 1000000*4;
+	tmp = (int)ceil((1000.0 * 1000.0 * inp_alsa_var_frames_target) / inp_alsa_var_samplerate);  // in micro seconds
 	if (snd_pcm_hw_params_set_buffer_time_near(priv->chandle, hwparams, &tmp, &dir) < 0){
 		visual_log(VISUAL_LOG_CRITICAL, _("Error setting buffer time"));
 		snd_pcm_hw_params_free(hwparams);
@@ -221,7 +221,8 @@ int inp_alsa_upload (VisPluginData *plugin, VisAudio *audio)
 {
 	int16_t data[PCM_BUF_SIZE];
 	alsaPrivate *priv = NULL;
-	int rcnt;
+	const snd_pcm_uframes_t frames_wanted = sizeof(data) / sizeof(data[0]) / inp_alsa_var_channels;
+	snd_pcm_sframes_t rcnt;
 	int i;
 
 	visual_log_return_val_if_fail(audio != NULL, -1);
@@ -229,6 +230,10 @@ int inp_alsa_upload (VisPluginData *plugin, VisAudio *audio)
 	priv = visual_object_get_private (VISUAL_OBJECT (plugin));
 	visual_log_return_val_if_fail(priv != NULL, -1);
 
+	if (priv->chandle == NULL) {
+		return -1;
+	}
+
 #if 0
 	{	/* DEBUG STUFF, REMOVE IN RELEASE FIXME FIXME XXX TODO WHATEVER */
 		VisBuffer buffer;
@@ -243,12 +248,12 @@ int inp_alsa_upload (VisPluginData *plugin, VisAudio *audio)
 	}
 #endif
 	do {
-		rcnt = snd_pcm_readi(priv->chandle, data, PCM_BUF_SIZE / 2);
+		rcnt = snd_pcm_readi(priv->chandle, data, frames_wanted);
 
 		if (rcnt > 0) {
 			VisBuffer buffer;
 
-			visual_buffer_init (&buffer, data, rcnt, NULL);
+			visual_buffer_init (&buffer, data, rcnt * sizeof(data[0]) * inp_alsa_var_channels, NULL);
 
 			visual_audio_samplepool_input (audio->samplepool, &buffer, VISUAL_AUDIO_SAMPLE_RATE_44100,
 					VISUAL_AUDIO_SAMPLE_FORMAT_S16, VISUAL_AUDIO_SAMPLE_CHANNEL_STEREO);
